@@ -191,12 +191,32 @@ class ClientBase(object):
         action = self.action_prefix + action
         return action
 
+    def _build_params_query(self, params=None):
+        flag_params = []
+        keyval_params = {}
+        for key, value in params.items():
+            if value is None:
+                flag_params.append(key)
+            else:
+                keyval_params[key] = value
+
+        flags_encoded = utils.safe_encode_list(flag_params) \
+            if flag_params else ""
+        keyval_encoded = utils.safe_encode_dict(keyval_params) \
+            if keyval_params else ""
+
+        query = ""
+        for flag in flags_encoded:
+            query = query + urlparse.quote_plus(flag) + '&'
+        query = query + urlparse.urlencode(keyval_encoded, doseq=1)
+        return query.strip('&')
+
     def do_request(self, method, action, body=None, headers=None, params=None):
         action = self.build_action(action)
         # Add format and tenant_id
         if type(params) is dict and params:
-            params = utils.safe_encode_dict(params)
-            action += '?' + urlparse.urlencode(params, doseq=1)
+            query = self._build_params_query(params)
+            action += '?' + query
 
         if body or body == {}:
             body = self.serialize(body)
@@ -205,8 +225,11 @@ class ClientBase(object):
             action, method, body=body,
             content_type=self.content_type())
 
-        if ('application/json' in resp.headers.get('Content-Type',
-                                                   'application/json')):
+        if 'application/zip' == resp.headers.get('Content-Type'):
+            self.format = 'zip'
+        elif 'text/plain' == resp.headers.get('Content-Type'):
+            self.format = 'text'
+        else:
             self.format = 'json'
 
         status_code = resp.status_code
@@ -231,7 +254,7 @@ class ClientBase(object):
         """
         if data is None:
             return None
-        elif self.format == 'zip':
+        elif self.format in ('zip', 'text'):
             return data
         elif type(data) is dict:
             return serializer.Serializer(
@@ -242,7 +265,7 @@ class ClientBase(object):
 
     def deserialize(self, data, status_code):
         """Deserializes an XML or JSON string into a dictionary."""
-        if status_code in (204, 202):
+        if status_code in (204, 202) or self.format in ('zip', 'text'):
             return data
         return serializer.Serializer(self.get_attr_metadata()).deserialize(
             data, self.content_type())['body']
@@ -261,12 +284,17 @@ class ClientBase(object):
                 constants.EXT_NS: ns}
 
     def content_type(self, _format=None):
-        """Returns the mime-type for either 'xml' or 'json'.
+        """Returns the mime-type for either 'xml', 'json, 'text', or 'zip'.
 
         Defaults to the currently set format.
         """
         _format = _format or self.format
-        return "application/%s" % (_format)
+        if self.format == 'text':
+            return "text/plain"
+        elif self.format == 'both':
+            return "text/plain,application/zip"
+        else:
+            return "application/%s" % (_format)
 
     def retry_request(self, method, action, body=None,
                       headers=None, params=None):
@@ -313,12 +341,19 @@ class ClientBase(object):
         return self.retry_request("PUT", action, body=body,
                                   headers=headers, params=params)
 
+    def patch(self, action, body=None, headers=None, params=None):
+        return self.retry_request("PATCH", action, body=body,
+                                  headers=headers, params=params)
+
     def list(self, collection, path, retrieve_all=True, **params):
         if retrieve_all:
             res = []
             for r in self._pagination(collection, path, **params):
-                res.extend(r[collection])
-            return {collection: res}
+                if type(r) is list:
+                    res.extend(r)
+                else:
+                    res.extend(r[collection])
+            return {collection: res} if collection else res
         else:
             return self._pagination(collection, path, **params)
 
@@ -333,6 +368,11 @@ class ClientBase(object):
             yield res
             next = False
             try:
+                # TODO(tpatil): Handle pagination for list data type
+                # once it's supported by tacker.
+                if type(res) is list:
+                    break
+
                 for link in res['%s_links' % collection]:
                     if link['rel'] == linkrel:
                         query_str = urlparse.urlparse(link['href']).query
@@ -733,6 +773,8 @@ class VnfPackageClient(ClientBase):
 
     vnfpackages_path = '/vnfpkgm/v1/vnf_packages'
     vnfpackage_path = '/vnfpkgm/v1/vnf_packages/%s'
+    vnfpackage_vnfd_path = '/vnfpkgm/v1/vnf_packages/%s/vnfd'
+    vnfpackage_download_path = '/vnfpkgm/v1/vnf_packages/%s/package_content'
 
     def build_action(self, action):
         return action
@@ -773,6 +815,87 @@ class VnfPackageClient(ClientBase):
                 base_path=self.vnfpackages_path),
                 body=file_data)
 
+    @APIParamsCall
+    def download_vnf_package(self, vnf_package):
+        self.format = 'zip'
+        return self.get(self.vnfpackage_download_path % vnf_package)
+
+    @APIParamsCall
+    def download_vnfd_from_vnf_package(self, vnf_package, accept):
+        """Read VNFD of an on-boarded VNF Package.
+
+        :param vnf_package: The value can be either the ID of a vnf package
+                            or a :class:`~openstack.nfv_orchestration.v1.
+                            vnf_package` instance.
+        :param accept: Valid values are 'text/plain', 'application/zip' and
+                       'both'. According to these values 'Accept' header will
+                        be set as 'text/plain', 'application/zip',
+                       'text/plain,application/zip' respectively.
+
+        :returns: If the VNFD is implemented in the form of multiple files,
+                  a ZIP file embedding these files shall be returned.
+                  If the VNFD is implemented as a single file, either that
+                  file or a ZIP file embedding that file shall be returned.
+        """
+        if accept == 'text/plain':
+            self.format = 'text'
+        elif accept == 'application/zip':
+            self.format = 'zip'
+        else:
+            self.format = 'both'
+        return self.get(self.vnfpackage_vnfd_path % vnf_package)
+
+    @APIParamsCall
+    def update_vnf_package(self, vnf_package, body):
+        return self.patch(self.vnfpackage_path % vnf_package, body=body)
+
+
+class VnfLCMClient(ClientBase):
+    """Client for vnflcm APIs.
+
+    Purpose of this class is to create required request url for vnflcm
+    APIs.
+    """
+
+    vnf_instances_path = '/vnflcm/v1/vnf_instances'
+    vnf_instance_path = '/vnflcm/v1/vnf_instances/%s'
+
+    def build_action(self, action):
+        return action
+
+    @APIParamsCall
+    def create_vnf_instance(self, body):
+        return self.post(self.vnf_instances_path, body=body)
+
+    @APIParamsCall
+    def show_vnf_instance(self, vnf_id, **_params):
+        return self.get(self.vnf_instance_path % vnf_id, params=_params)
+
+    @APIParamsCall
+    def list_vnf_instances(self, retrieve_all=True, **_params):
+        vnf_instances = self.list(None, self.vnf_instances_path,
+                                  retrieve_all, **_params)
+        return vnf_instances
+
+    @APIParamsCall
+    def instantiate_vnf_instance(self, vnf_id, body):
+        return self.post((self.vnf_instance_path + "/instantiate") % vnf_id,
+                         body=body)
+
+    @APIParamsCall
+    def heal_vnf_instance(self, vnf_id, body):
+        return self.post((self.vnf_instance_path + "/heal") % vnf_id,
+                         body=body)
+
+    @APIParamsCall
+    def terminate_vnf_instance(self, vnf_id, body):
+        return self.post((self.vnf_instance_path + "/terminate") % vnf_id,
+                         body=body)
+
+    @APIParamsCall
+    def delete_vnf_instance(self, vnf_id):
+        return self.delete(self.vnf_instance_path % vnf_id)
+
 
 class Client(object):
     """Unified interface to interact with multiple applications of tacker service.
@@ -794,6 +917,7 @@ class Client(object):
     """
 
     def __init__(self, **kwargs):
+        self.vnf_lcm_client = VnfLCMClient(**kwargs)
         self.vnf_package_client = VnfPackageClient(**kwargs)
         self.legacy_client = LegacyClient(**kwargs)
 
@@ -1005,7 +1129,8 @@ class Client(object):
     def create_vnf_package(self, body):
         return self.vnf_package_client.create_vnf_package(body)
 
-    def list_vnf_packages(self, retrieve_all=True, **_params):
+    def list_vnf_packages(self, retrieve_all=True, query_parameter=None,
+                          **_params):
         return self.vnf_package_client.list_vnf_packages(
             retrieve_all=retrieve_all, **_params)
 
@@ -1018,3 +1143,38 @@ class Client(object):
 
     def delete_vnf_package(self, vnf_package):
         return self.vnf_package_client.delete_vnf_package(vnf_package)
+
+    # VnfLCMClient methods.
+
+    def create_vnf_instance(self, body):
+        return self.vnf_lcm_client.create_vnf_instance(body)
+
+    def show_vnf_instance(self, vnf_instance, **_params):
+        return self.vnf_lcm_client.show_vnf_instance(vnf_instance,
+                                                     **_params)
+
+    def list_vnf_instances(self, retrieve_all=True, **_params):
+        return self.vnf_lcm_client.list_vnf_instances(
+            retrieve_all=retrieve_all, **_params)
+
+    def instantiate_vnf_instance(self, vnf_id, body):
+        return self.vnf_lcm_client.instantiate_vnf_instance(vnf_id, body)
+
+    def heal_vnf_instance(self, vnf_id, body):
+        return self.vnf_lcm_client.heal_vnf_instance(vnf_id, body)
+
+    def terminate_vnf_instance(self, vnf_id, body):
+        return self.vnf_lcm_client.terminate_vnf_instance(vnf_id, body)
+
+    def delete_vnf_instance(self, vnf_id):
+        return self.vnf_lcm_client.delete_vnf_instance(vnf_id)
+
+    def update_vnf_package(self, vnf_package, body):
+        return self.vnf_package_client.update_vnf_package(vnf_package, body)
+
+    def download_vnfd_from_vnf_package(self, vnf_package, accept):
+        return self.vnf_package_client.download_vnfd_from_vnf_package(
+            vnf_package, accept)
+
+    def download_vnf_package(self, vnf_package):
+        return self.vnf_package_client.download_vnf_package(vnf_package)
